#import "AFSControl.h"

@implementation AFSControl

/* When app is first run */
- (void)awakeFromNib {

    /* Privileged app stuff */
    char * mypath;
    OSStatus myStatus;

    /* Hopefully, we can grab the privs */
    myStatus = AuthorizationCopyPrivilegedReference(&myAuthorizationRef, kAuthorizationFlagDefaults);

    if (myStatus != errAuthorizationSuccess) { /* Need to get authorization, then rerun ourself */
 
        AuthorizationFlags myFlags = kAuthorizationFlagDefaults;

        /* Create auth object */
        myStatus = AuthorizationCreate(NULL, kAuthorizationEmptyEnvironment, myFlags, &myAuthorizationRef);
        if (myStatus != errAuthorizationSuccess) {
            exit(1);
        }
        
        do 
        {
            {
                /* Get path to me */
                mypath = malloc(MAXPATHLEN);
                unsigned long * mylength;
                _NSGetExecutablePath(mypath, mylength);
                
                /* Get rights */
                AuthorizationItem myItems = {kAuthorizationRightExecute, 0, NULL, 0};
                AuthorizationRights myRights = {1, &myItems};

                myFlags = kAuthorizationFlagDefaults | kAuthorizationFlagInteractionAllowed | kAuthorizationFlagPreAuthorize |                    kAuthorizationFlagExtendRights;
                myStatus = AuthorizationCopyRights (myAuthorizationRef, &myRights, NULL, myFlags, NULL );
            }
           
            if (myStatus != errAuthorizationSuccess) {
                AuthorizationFree (myAuthorizationRef, kAuthorizationFlagDefaults);
                exit(1);
            }
         
            {
                /* Prepare for calling self */
                char *myArguments[] = { NULL, NULL};
                //char *myArguments[] = { argv[1], NULL };
                FILE *myCommunicationsPipe = NULL;
                char myReadBuffer[128];

                myFlags = kAuthorizationFlagDefaults;
                myStatus = AuthorizationExecuteWithPrivileges (myAuthorizationRef, mypath, myFlags, myArguments, &myCommunicationsPipe);

                /* Call self if successful auth */
                if (myStatus == errAuthorizationSuccess)
                for(;;)
                {
                    int bytesRead = read (fileno (myCommunicationsPipe), myReadBuffer, sizeof (myReadBuffer));
                    if (bytesRead < 1) break;
                    write (fileno (stdout), myReadBuffer, bytesRead);
                }
            }
        } while (0);
        exit(1);
    } /* End if no privs */

    //AuthorizationFree (myAuthorizationRef, kAuthorizationFlagDefaults);                //17


    /* End from runwithprivs */

    else {


    
        /* Bring up first view */
        [BoxOutlet setContentView:ThisCellViewOutlet];
        [MainWindowOutlet makeKeyAndOrderFront:self];
        [MainWindowOutlet orderFrontRegardless];
        [NSApp activateIgnoringOtherApps:YES];

    }
}

/* Finished confirm sheet */
- (IBAction)FinishSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo {

    if (returnCode == NSAlertDefaultReturn) {

        /* Get data */
        NSString * ThisCell = [CellNameOutlet stringValue];
        NSString * CellAlias = [CellAliasOutlet stringValue];
        NSString * CellServ = [[CellServOutlet textStorage] string];
        NSString * AfsOptions = [OptionsOutlet stringValue];
        
        /* Write data to files */
        [ThisCell writeToFile:@"/var/db/openafs/etc/ThisCell" atomically:YES];
        [CellServ writeToFile:@"/var/db/openafs/etc/CellServDB" atomically:YES];
        [AfsOptions writeToFile:@"/var/db/openafs/etc/config/afsd.options" atomically:YES];
        
        if ([CellAlias length] != 0) {
            NSMutableString * AliasToWrite = [[NSMutableString alloc] init];
            [AliasToWrite appendString:ThisCell];
            [AliasToWrite appendString:@" "];
            [AliasToWrite appendString:CellAlias];
            [AliasToWrite writeToFile:@"/var/db/openafs/etc/CellAlias" atomically:YES];
        }
        


        exit(1);
    }
}

/* Quit confirm sheet */
- (IBAction)QuitSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo {

    if (returnCode == NSAlertDefaultReturn) {
        exit(0);
    }
}

/* Go to next step */
- (IBAction)NextAction:(id)sender
{
    if ([BoxOutlet contentView] == ThisCellViewOutlet) { /* Next */
    
        /* Check CellName to not be empty */
        if ([[CellNameOutlet stringValue] length] == 0) {
            NSBeginAlertSheet(@"Cell Name", @"Fix", nil, nil, MainWindowOutlet, self, nil,  nil, nil, @"You forgot to give a Cell Name.");
        
        }
        
        else {

            if ([[CellServOutlet textStorage] length] == 0) { /* Fill with contents of CellServDb on disk */
                NSString * CSFromFile = [[NSString alloc] initWithContentsOfFile:@"/var/db/openafs/etc/CellServDB"];
                NSAttributedString * CSData = [[NSAttributedString alloc] initWithString:CSFromFile];
                [[CellServOutlet textStorage] setAttributedString:CSData];
                
                [CSFromFile autorelease];
                [CSData autorelease];
            }
            
            [BoxOutlet setContentView:CellServViewOutlet];
        }
    }

    else if ([BoxOutlet contentView] == CellServViewOutlet) { /* Next */
    
        if ([[OptionsOutlet stringValue] length] == 0) { /* Set default value */
            [OptionsOutlet setStringValue:@"-fakestat -afsdb -dynroot -nosettime -stat 2000 -dcache 800 -daemons 3 -volumes 70"];
        }

        [BoxOutlet setContentView:OptionsViewOutlet];
        [NextButtonOutlet setTitle:@"Finished"];

    }
    
    else { /* Finish pressed */
    
            NSBeginAlertSheet(@"Finished?", @"Done", @"Not Done", nil, MainWindowOutlet, self, nil, @selector(FinishSheetDidEnd:returnCode:contextInfo:), nil, @"Are you done making changes?");
    
    }
}

/* Go to previous step */
- (IBAction)PreviousAction:(id)sender
{

    if ([BoxOutlet contentView] == OptionsViewOutlet) { /* Previous */
        [NextButtonOutlet setTitle:@"Next Step"];
        [BoxOutlet setContentView:CellServViewOutlet];
    }
    
    else if ([BoxOutlet contentView] == CellServViewOutlet) { /* Previous */
        [BoxOutlet setContentView:ThisCellViewOutlet];
    }
}

/* Quit button pressed */
- (IBAction)QuitAction:(id)sender
{
    NSBeginAlertSheet(@"Quit", @"Quit", @"Don't Quit", nil, MainWindowOutlet, self, nil, @selector(QuitSheetDidEnd:returnCode:contextInfo:), nil, @"Are you sure you want to quit? You will lose any changes you have made. And, AFS may not work properly.");


}

@end
